package com.example.ueberwachungssystem;

import static java.lang.Math.sqrt;

import android.content.Context;
import android.hardware.Sensor;
import android.hardware.SensorEvent;
import android.hardware.SensorEventListener;
import android.hardware.SensorManager;
import android.widget.TextView;

import androidx.appcompat.app.AppCompatActivity;

public class Accelerometer extends Detector implements SensorEventListener {

    Logger logger;

    private volatile boolean running = false;
    public SensorManager sensorManager;
    private static final int sensorType = Sensor.TYPE_LINEAR_ACCELERATION;
    private Sensor accelerometer;
    private Context context;

    private String threadName = "Accelerometer Thread";
    boolean alarm = false;
    //Preallocate memory for the data of each axis of the acceleration sensor
    float x;
    float y;
    float z;
    float betrag; //Betrag aller drei Achsen sqrt(x*x + y*y + z*z)
    private TextView textViewWorkerThread;
    private AppCompatActivity activity;
    private DetectionReport detectionReport;


    // In constructor pass Activity, Context and TextView from MainActivity in Accelerometer class
    public Accelerometer(Context context, Logger mainLogger, TextView textViewWorkerThread){
        super(context); //von Detektor
        logger = mainLogger;
        this.context = context;
        this.textViewWorkerThread = textViewWorkerThread;
        Logger logger = new Logger(this.getClass().getSimpleName(), textViewWorkerThread, "");
    }

    public void getSensor(){
        sensorManager = (SensorManager)context.getSystemService(Context.SENSOR_SERVICE);
        if(sensorManager.getSensorList(sensorType).size()==0) {
            logger.log("Es gibt den gewünschten Sensor nicht");
            accelerometer = null;
        }
        else {
            accelerometer = sensorManager.getSensorList(sensorType).get(0);
            logger.log("Sensor gefunden");
        }
    }


    void start() {
        // register the sensor before using //
        sensorManager.registerListener(this, accelerometer, SensorManager.SENSOR_DELAY_GAME);
        logger.log("Accelerometer, start!");
    }


    void stop() {
        // unregister from the sensor to stop using it //
        sensorManager.unregisterListener(this, accelerometer);
        logger.log("Accelerometer unregistered!");
    }


    @Override
    public void onSensorChanged(SensorEvent event) {
        checkAlarm(event);
    }

    StringBuffer stringBuffer = new StringBuffer();
    public void checkAlarm (SensorEvent event) {
        x = event.values[0];
        y = event.values[1];
        z = event.values[2];
        betrag = (float) sqrt(x*x + y*y + z*z);
        double schwelle = 1.5;
        //stringBuffer.append("\nBetrag =" + betrag);
        //logger.log("Betrag" + betrag);

        if (!alarm) {
            if (betrag > schwelle) {
                stringBuffer.append("\n Betragswert über Schwelle, Detection Report wird gleich aufgerufen");
                //logger.log("Betragswert über Schwelle erkannt, Alarm wird gleich angeschaltet");
                alarm = true;
                stringBuffer.append("\nAlarm = " + alarm);
                detectionReport = new DetectionReport("Accelerometer1", "Bewegung", betrag);
                stringBuffer.append("\nDetectionReport = " + detectionReport)
                            .append("\nAlarm an" + betrag + event.timestamp);
                //logger.log("Alarm an");
                //logger.log("Betrag: " + betrag + event.timestamp);
                logger.clearLog();
                logger.log(stringBuffer.toString());
                stringBuffer.delete(0, stringBuffer.length());
            }
        } else {
            if (betrag < schwelle) {
                stringBuffer.append("\nAlarm noch an; Neuer Betragswert unter Schwellwert:" + betrag);
                //logger.log("Alarm ist noch an; Neuer Betragswert unter Schwellwert: " + betrag);
                alarm = false;
                //logger.log("Alarm" + alarm);
                //logger.log("Alarm wieder ausgeschaltet");
                stringBuffer.append("\nAlarm = " + alarm);
            } else {
                logger.log("Betragswert immer noch über Schwellwert: " + betrag + "; Alarm bleibt an.");
                logger.log("Betragswert immer noch über Schwellwert: " + betrag + "; Alarm bleibt an.");
            }
            logger.clearLog();
            logger.log(stringBuffer.toString());
            stringBuffer.delete(0, stringBuffer.length());
        }
    }

    @Override
    public void onAccuracyChanged(Sensor sensor, int accuracy) {
    }

    @Override
    public void startDetection() {
        // entspricht void start()
        if (accelerometer != null) {
            sensorManager.registerListener(this, accelerometer, SensorManager.SENSOR_DELAY_GAME);
            logger.log("Sensor registriert");
        }
    }

    @Override
    public void stopDetection() {
        // entspricht void stop()
        sensorManager.unregisterListener(this, accelerometer);
        logger.log("Vom Sensor abgemeldet");
    }
}